from jeffsSprite import *
import pygame, sys, random

class Subtitle():
    def __init__(self, line, shaky = 0, unstability = 1, color = (255, 255, 255), preset = 1, fontSize = 30, life = 60, slowness = 2, id=0):
        self.line = line
        self.shaky = shaky
        self.unstability  = unstability
        self.color = color
        self.preset = preset
        self.fontSize = fontSize
        self.life = life
        self.slowness = slowness
        self.id = id

class ChatBox(PointSprite):
    def __init__(self, screen):
        PointSprite.__init__(self)
        self.screen = screen
        self.length = 10
        self.center = (25, 25)
        self.color = (255, 255, 255)
        self.w = 350
        self.h = 80
        self.unstability = 4
        self.pw = self.w/self.length
        self.ph = self.h/self.length
        self.points.extend([(self.length * i,0) for i in range(self.pw)])
        self.points.extend([(self.w,self.length * i) for i in range(self.ph)])
        self.points.extend([(self.length * (self.pw - i),self.h) for i in range(self.pw)])
        self.points.extend([(0,self.length * (self.ph - i)) for i in range(self.ph)])
        self.offsets = [(0, 0) for i in range(len(self.points))]
        self.tick = 0
        self.slowness = 2
        self.presets = ("chakra.ttf", "shadows.ttf")
        self.font = pygame.font.Font("chakra.ttf", 30)
        self.fontSize = 0
        self.lines = []
        self.subtitles = []
        self.shaky = 0
        self.life = 0
        self.id = 0

    def get_points(self):
        return [overlay(overlay(self.points[i], self.center), self.offsets[i]) for i in range(len(self.points))]

    def draw(self):
        if self.lines:
            pygame.draw.polygon(self.screen, self.color, self.get_points(), 3)
            for i, line in enumerate(self.lines):
                self.screen.blit(self.font.render(line, False, self.color),
                                 (self.center[0]+self.unstability*2+1+random.randint(-self.shaky, self.shaky),
                                  self.center[1] + i * self.fontSize+random.randint(-self.shaky, self.shaky)))

    def addSub(self, subtitle):
        self.subtitles.append(subtitle)

    def add(self, subtitle):
        self.tick = 0
        self.id = subtitle.id
        self.lines = []
        self.shaky = subtitle.shaky
        self.unstability = subtitle.unstability
        self.color = subtitle.color
        self.font = pygame.font.Font(self.presets[subtitle.preset], subtitle.fontSize)
        self.fontSize = subtitle.fontSize
        self.life = subtitle.life
        self.slowness = subtitle.slowness

        currentString = ''
        for char in subtitle.line:
            currentString += char
            if char == '&':
                self.lines.append(currentString[:-1])
                currentString = ''
            elif self.font.size(currentString)[0] > self.w-self.unstability*4:
                self.lines.append(currentString[:-1])
                currentString = currentString[-1]
        if currentString != '':
            self.lines.append(currentString)

    def clear(self):
        self.subtitles = []
        self.lines = []

    def update(self):
        value = 0
        if self.lines:
            self.tick += 1
            if self.tick > self.life:
                self.lines = []
            elif self.tick % 2:
                for i in range(len(self.offsets)):
                    self.offsets[i] = (random.randint(-self.unstability,self.unstability), random.randint(-self.unstability,self.unstability))
        else:
            value = self.id
            self.id = 0
            if self.subtitles:
                self.add(self.subtitles.pop(0))

        return value

if __name__ == "__main__":
    pygame.init()
    screen = pygame.display.set_mode((400, 400))
    box = ChatBox(screen)
    clock = pygame.time.Clock()
    box.addSub(Subtitle("Hello world! Hello&pyweek!"))
    box.addSub(Subtitle("Second line!"))
    box.addSub(Subtitle("Third line!"))
    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
        box.update()
        screen.fill(0)
        box.draw()
        pygame.display.flip()
        clock.tick(20)
