from jeffsSprite import *
import pygame

class Seed():
    def __init__(self, angleRange, lengthRange, amountRange, decayRange):
        self.angleRange = angleRange
        self.lengthRange = lengthRange
        self.amountRange = amountRange
        self.decayRange = decayRange

    def decay(self):
        decay = getRandom(self.decayRange)
        tempPit = self.lengthRange[0] - decay
        tempPeak = self.lengthRange[1] - decay
        if tempPit < 0:
            tempPit = 0
        if tempPeak < 0:
            tempPeak = 0
        if tempPeak == 0:
            return
        self.lengthRange = (tempPit, tempPeak)

class Branch(PointSprite):
    def __init__(self, angle, length, growAmount, screen, parent = None):
        PointSprite.__init__(self)
        self.children = []
        self.screen = screen
        self.grew = False
        self.angle = angle
        self.length = length
        if parent:
            self.points = parent.getPoints(growAmount)
        else:
            self.points = [(0,0), growAmount]
        self.parent = parent

    def getPoints(self, growAmount):
        return [self.points[1], overlay(self.points[1], growAmount)]

    def draw(self):
        self.update()
        pygame.draw.line(self.screen, -1, self.points[0], self.points[1], 2)
        for child in self.children:
            child.draw()

    def move(self, x, y):
        self.points[0] = overlay(self.points[0], (x,y))
        self.points[1] = overlay(self.points[1], (x,y))
        for child in self.children:
            child.move(x, y)

    def set(self, x, y):
        if x != self.points[0][0] or y != self.points[0][1]:
            dx = x - self.points[0][0]
            dy = y - self.points[0][1]
            self.move(dx, dy)

    def rotate(self, angle):
        tempx, tempy = self.points[0][0], self.points[0][1]
        tempPoint = (self.points[1][0]-self.points[0][0], self.points[1][1]-self.points[0][1])
        tempPoint2 = rotate(tempPoint, angle)
        dx, dy = tempPoint2[0]-tempPoint[0], tempPoint2[1]-tempPoint[1]
        self.move(dx, dy)
        self.points[0] = (tempx, tempy)

class Tree():
    def __init__(self, seed, basePos, screen):
        self.screen = screen
        self.seed = seed
        self.baseHeight = getRandom(seed.lengthRange)
        self.baseBranch = [Branch(math.pi/2, self.baseHeight, (0, self.baseHeight), self.screen)]
        self.baseBranch[0].move(basePos[0], basePos[1])
        self.basePos = basePos
        self.rect = pygame.Rect(self.basePos[0]-10, self.basePos[1]+self.baseHeight-10, 20, -self.baseHeight+20)
        self.hovered = False

    def grow(self):
        newBranches = []
        for branch in self.baseBranch:
            if not branch.grew:
                numbers = getRandom(self.seed.amountRange)
                for i in range(numbers):
                    angle = getRandFloat(self.seed.angleRange)+branch.angle
                    length = getRandom(self.seed.lengthRange)
                    newBranch = Branch(angle, length, radarToVec(length, angle), self.screen, branch)
                    branch.children.append(newBranch)
                    newBranches.append(newBranch)
                    speed = length*length/2000000.0
                    newBranch.addRepeatingEvent(2, 72, speed)

                    newBranch.addRepeatingEvent(2, 18, speed/2)

                    newBranch.addRepeatingEvent(2, 18, -speed/2)

                    newBranch.addRepeatingEvent(2, 144, -speed)

                    newBranch.addRepeatingEvent(2, 18, -speed/2)

                    newBranch.addRepeatingEvent(2, 72, speed)

                    newBranch.addRepeatingEvent(2, 18, speed/2)
                branch.grew = True
        self.baseBranch.extend(newBranches)
        self.seed.decay()

    def draw(self):
        self.baseBranch[0].draw()

    def update(self):
        pass

if __name__ == "__main__":
    screen = pygame.display.set_mode((400, 400))
    clock = pygame.time.Clock()
    tree = Tree(Seed((-math.pi/6, math.pi/6), (-50,-30), (2,3), (2,3)), (200, 400), screen)
    tree.grow()
    tree.grow()
    tree.grow()
    tree.grow()
    while True:
        screen.fill((0, 0, 0))
        tree.draw()
        pygame.display.flip()
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
        clock.tick(30)