from jeffsMath import *

#Event id list: {1:move, 2:rotate, 3:enlarge, 4:set velocity}
class Event():
    def __init__(self, id, duration, target, *args):
        self.id = id
        self.duration = duration
        self.lasted = 0
        self.target = target
        self.para = args

    def tick(self):
        if self.id == 1:
            self.target.move(self.para[0], self.para[1])
        elif self.id == 2:
            self.target.rotate(self.para[0])
        elif self.id == 3:
            self.target.resize(self.para[0])
        elif self.id == 4:
            self.target.setVelocity(self.para[0], self.para[1])
        self.lasted += 1
        if self.lasted >= self.duration:
            return False
        return True

class Sprite():
    def __init__(self):
        self.pos = [random.randint(0, 400), random.randint(0, 400)]
        self.length = random.randint(3, 6)
        self.actions = []
        self.repeatingActions = []
        self.angle = 0
        self.center = [self.pos[0], self.pos[1]]
        self.vel = [0,0]

    def update(self):
        self.move(self.vel[0], self.vel[1])
        if self.actions:
            if not self.actions[0].tick():
                del self.actions[0]
        if self.repeatingActions:
            if not self.repeatingActions[0].tick():
                self.repeatingActions[0].lasted = 0
                self.repeatingActions.append(self.repeatingActions.pop(0))

    def draw(self):
        pass

    def addEvent(self, id, duration, *args):
        self.actions.append(Event(id, duration, self, *args))

    def addRepeatingEvent(self, id, duration, *args):
        self.repeatingActions.append(Event(id, duration, self, *args))

    def set(self, x, y):
        self.pos[0] = x
        self.pos[1] = y
        self.center[0] = x
        self.center[1] = y

    def move(self, x, y):
        self.pos[0] += x
        self.pos[1] += y
        self.center[0] += x
        self.center[1] += y

    def rotate(self, angle):
        self.angle += angle

    def resize(self, ratio):
        self.length *= ratio

    def setVelocity(self, x, y):
        self.vel[0] = x
        self.vel[1] = y

class PointSprite(Sprite):
    def __init__(self):
        Sprite.__init__(self)
        self.points = []

    def rotate(self, angle):
        Sprite.rotate(self, angle)
        for i in range(len(self.points)):
            self.points[i] = rotate(self.points[i], angle)

    def resize(self, ratio):
        Sprite.resize(self, ratio)
        for i in range(len(self.points)):
            self.points[i] = enlarge(self.points[i], ratio)

    def get_points(self):
        return [overlay(point, self.center) for point in self.points]


class Controllable(Sprite):
    def __init__(self):
        Sprite.__init__(self)

