import pygame
from noise import pnoise1

class Terrain2D():
    def __init__(self, start, end, gap, screen, yLevel = 300, yLimit = 20):
        self.pointPositions = range(start, end+gap, gap)
        self.pointHeights = [0 for i in (self.pointPositions)]
        self.start = start
        self.end = end
        self.gap = gap
        self.screen = screen
        self.length = (self.end-self.start)/self.gap
        self.yLevel = yLevel
        self.yLimit = yLimit

    def __len__(self):
        return self.length

    def capHeight(self, index):
        if self.pointHeights[index] > self.yLimit:
            self.pointHeights[index] = self.yLimit
        elif self.pointHeights[index] < -self.yLimit:
            self.pointHeights[index] = -self.yLimit

    def raisePosition(self, x, value):
        if self.end > x > self.start:
            index = int((x-self.start)/self.gap)
            self.pointHeights[index] += value
            self.pointHeights[index+1] += value
            self.capHeight(index)
            self.capHeight(index+1)
            return True
        return False

    def getPoint(self, index):
        return (self.pointPositions[index], self.pointHeights[index]+self.yLevel)

    def getHeight(self, position):
        if self.end > position > self.start:
            index = int((position - self.start) / self.gap)
            difference = self.pointHeights[index+1] - self.pointHeights[index]
            return ((position-self.start)%self.gap)/float(self.gap)*difference+self.pointHeights[index]+self.yLevel

    def draw(self):
        for i in range(self.length):
            pygame.draw.line(self.screen, (255,255,255), self.getPoint(i), self.getPoint(i+1), 3)

    def handleEvent(self, event, mousePos):
        value = False
        if event.type == pygame.QUIT:
            pygame.quit()
        elif event.type == pygame.MOUSEBUTTONDOWN:
            if event.button == 4:
                self.raisePosition(mousePos[0], -3)
                value = True
            elif event.button == 5:
                self.raisePosition(mousePos[0], 3)
                value = True
        return value

if __name__ == "__main__":
    screen = pygame.display.set_mode((400, 400))
    clock = pygame.time.Clock()
    terrain = Terrain2D(0, 400, 20, screen)
    mousePos = pygame.mouse.get_pos()
    while True:
        screen.fill((0, 0, 0))
        terrain.draw()
        pygame.display.flip()
        for event in pygame.event.get():
            if event.type == pygame.MOUSEMOTION:
                mousePos = pygame.mouse.get_pos()
            terrain.handleEvent(event, mousePos)
        clock.tick(30)
