import pygame
from noise import pnoise1
import random
pygame.init()



pattern = 400.0

def getNoise(x, offset, fluctuate, scaleDown, octaves = 1):
    return pnoise1((x + offset/scaleDown) % (pattern/scaleDown) / (pattern/scaleDown), octaves, repeat=int(pattern/scaleDown)) * fluctuate

class Wave():
    def __init__(self, x, y, screen, length, gap, speed, depth, fluctuate = 20.0, color = (116,204,244)):
        self.x = x
        self.y = y
        self.depth = depth
        self.yOffset = random.randint(0,pattern)
        self.length = length
        self.gap = gap
        self.speed = speed
        self.pointNum = self.length/self.gap+1
        self.points = [[self.x+i*self.gap, self.y] for i in range(self.pointNum)]
        self.basePoints = ((self.x, self.y+self.depth), (self.x+self.pointNum*self.gap-self.gap, self.y+self.depth))
        self.fluctuate = fluctuate
        self.color = color
        self.screen = screen

    def generate(self):
        for i in range(self.pointNum):
            self.points[i][1] = self.y+getNoise(self.points[i][0], self.yOffset, self.fluctuate, 1)
            self.points[i][1] += getNoise(self.points[i][0], self.yOffset, self.fluctuate/2, -2)
            self.points[i][1] += getNoise(self.points[i][0], self.yOffset, self.fluctuate/4, 1, 3)

    def draw(self):
        self.generate()
        self.yOffset += self.speed
        if self.yOffset > pattern:
            self.yOffset = 0
        tempPoints = [self.basePoints[1], self.basePoints[0]]
        for i in range(self.pointNum):
            tempPoints.append(self.points[i])
        #for i in range(self.pointNum-1):
        #    pygame.draw.line(screen, (255,255,255), self.points[i], self.points[i+1])
        pygame.draw.polygon(self.screen, self.color, tempPoints)

if __name__ == "__main__":
    screen = pygame.display.set_mode((400, 400))
    clock = pygame.time.Clock()
    wave = Wave(50,250,screen,300,10,5,100)
    wave.generate()

    while True:
        screen.fill((0, 0, 0))
        wave.draw()
        pygame.display.flip()
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
        clock.tick(30)
